//
//  Angle.swift
//  HexagonTiledTorus
//
//  Created by Don McBrien on 22/02/2020.
//  Copyright © 2020 Don McBrien. All rights reserved.
//

import Foundation

public let π = Float.pi

public struct Angle: CustomStringConvertible {
   public static func * (lhs: Angle, rhs: Float) -> Angle {
      return Angle(inDegrees: lhs.degrees * rhs)
   }

   public static func / (lhs: Angle, rhs: Float) -> Angle {
      return Angle(inDegrees: lhs.degrees / rhs)
   }

   public static func + (lhs: Angle, rhs: Angle) -> Angle {
      return Angle(inDegrees: lhs.degrees + rhs.degrees)
   }

   public static func - (lhs: Angle, rhs: Angle) -> Angle {
      return Angle(inDegrees: lhs.degrees - rhs.degrees)
   }

   // forced to integer number of degrees
   public let degrees : Float
   public var radians : Float { return Float(degrees) * π / 180.0 }
   
   public init(inRadians : Float ) {
      degrees = inRadians * 180.0 / π
   }
   
   public init(inDegrees : Float ) {
      degrees = inDegrees
   }
   
   public var description: String {
      return String(format: "% 6.2f° (= % 6.2f radians)", arguments: [degrees,radians])
   }
}
