//
//  Vertex.swift
//  HexagonTiledTorus
//
//  Created by Don McBrien on 01/03/2020.
//  Copyright © 2020 Don McBrien. All rights reserved.
//

import Foundation
import MetalKit
import simd

public struct Vertex: CustomStringConvertible {
   public let position: SIMD4<Float>
   public var colour: SIMD4<Float> = [0,0,1,1]

   public var description: String {
      return "position: [\(position.x),\(position.y),\(position.z),\(position.w)], colour: [\(colour.x),\(colour.y),\(colour.z),\(colour.w)]"
   }
   
   public static func - (lhs: Vertex, rhs: Vertex) -> SIMD3<Float> {
      return SIMD3<Float>([lhs.position.x - rhs.position.x,
                           lhs.position.y - rhs.position.y,
                           lhs.position.z - rhs.position.z])
   }
}

public typealias float4 = SIMD4<Float>

infix operator ×: ComparisonPrecedence

extension float4 {
   public static func ×(lhs:float4, rhs: float4) -> float4 {
      return float4(lhs.y * rhs.z - lhs.z * rhs.y,
                    lhs.z * rhs.x - lhs.x * rhs.z,
                    lhs.x * rhs.x - lhs.y * rhs.x, 1)
   }
   
   public static func /(lhs:float4, rhs: Float) -> float4 {
      return float4(lhs.x / rhs, lhs.y / rhs, lhs.z / rhs, 1)
   }
   
   public func normalised() -> float4 {
      let scalar = sqrt(self.x * self.x + self.z * self.z + self.z * self.z)
      return float4(self.x / scalar, self.y / scalar, self.z / scalar, 1)
   }

}

